function mit18086_fd_transport_limiter(method)
%MIT18086_FD_TRANSPORT_LIMITER
%    Solves the transport equation u_t+cu_x=0 by finite
%    difference methods. Of interest are discontinuous initial
%    conditions. Implemented methods are (1) upwind,
%    (2) Lax-Friedrichs, (3) Lax-Wendroff, (4) Lax-Wendroff
%    with van Leer flux limiter, and (5) Lax-Wendroff with
%    Superbee flux limiter.

% 03/2007 by Benjamin Seibold
%            http://www-math.mit.edu/~seibold/
% Feel free to modify for teaching and learning.

disp('        h           error          order')

n_loop = [100 200 400 800];
for  p=1:length(n_loop)
n = n_loop(p);

% n = 400;                           % number of space gridpoints
%dt = 4e-3;% time step
tf = 1;                                            % final time
c = 1;                                   % speed of transport
if nargin<1, method = 1; end
warning('off','MATLAB:divideByZero');
x = linspace(-1,1,n+1)';
h = x(2)-x(1);
u = ic(x); u0 = u;
dt= h;
%dt/h
nsteps=ceil(tf/dt);
nplot=max(nsteps/1000,1);
% figure(p)
 figure(1)
 subplot(2,2,p)
   plot(x,u0,'b:',x,u,'r.-')
   axis([-1 1 -.2 1.2])
   %title(sprintf('n=%0.0f, %s, t=%0.2f',n,name,tn*dt))
   %drawnow
   %input('Hit <return> to continue  ');
   pause(1)
for tn = 1:nsteps
%    ceil(tf/dt)
   theta = diffl(u)./(diffr(u)+3e-14);
   switch method
   case 1
      name = 'upwind';
      f = c*u;
   case 2
      name = 'Lax-Friedrichs';
      f = c*sumr(u)/2-h/2/dt*diffr(u);
   case 3
      name = 'Lax-Wendroff';
      f = c*sumr(u)/2-c^2*dt/2/h*diffr(u);
   case 4
      name = 'Lax-Wendroff von Leer';
      phi = limiter(theta,1);
      f = c*u+c/2*(1-c*dt/h)*diffr(u).*phi;
   case 5
      name = 'Lax-Wendroff Superbee';
      phi = limiter(theta,2);
      f = c*u+c/2*(1-c*dt/h)*diffr(u).*phi;
   end
   u = u-dt*diffl(f)/h;
   U = ic(x-c*tn*dt);
   %clf
% hold on
 if mod(tn,nplot)==0 | n==nsteps
%         uint = u(1:m+2);  % points on the interval (drop ghost cell on right)
%         ufine = utrue(xfine,tnp);
 %figure(p)
%  plot(x,uint,'b.-', xfine,ufine,'r')
%         axis([0 1 -1.2 1.2])
%         title(sprintf('t = %9.5e  after %4i time steps with %5i grid points',...
%                        tnp,n,m+1))
%         error = max(abs(uint-utrue(x,tnp)));
   plot(x,u0,'b:',x,U,'b-',x,u,'r.-')
   axis([-1 1 -.2 1.2])
   title(sprintf('n=%0.0f, %s, t=%0.2f',n,name,tn*dt))
   drawnow
   %input('Hit <return> to continue  ');
 end



% figure(6)
% plot3(x,tn,U,'r-')
end


E=abs(U-u);
Err(p)=norm(E,1);
if (p ==1)  
   order(p)=0;
else
    order(p)=log(Err(p)/Err(1))/log(n_loop(1)/n_loop(p));
end

   disp(sprintf('%13.4e   %13.4e  %13.4e',...
                 (2./n_loop(p)),Err(p),order(p)))

end



figure(5)
loglog(2./n_loop,Err,'d-')
xlabel('log(h)');
ylabel('log(Error)');



function y = diffl(x)
y = [0;diff(x)];
function y = diffr(x)
y = [diff(x);0];
function y = sumr(x)
y = [x(1:end-1)+x(2:end);2*x(end)];

function y = ic(x)
% initial condition function
y = (x>-.5&x<0)*0.0+(x>-1.0&x<-.5)*1;
% y=zeros(size(x));
% for i=1:size(x,1)
%    if (x(i)>=-.6) & (x(i)<=-.4)
%       % eta(i)=(x(i)-.6)^2*(x(i)-.4)^2*10^4;
%               y(i)=(x(i)+.6)^4*(x(i)+.4)^4*10^8;
%    else 
% 	eta(i)=0;
%    end
%   %eta(i)=sin(l*pi*x(i));
% end
% return



function phi = limiter(r,method)
% flux limiter
if nargin<1, method = 1; end
switch method
case 1
phi = (abs(r)+r)./(1+abs(r));                        % van Leer
case 2
phi = max(0,max(min(1,2*r),min(r,2)));               % Superbee
end


